import { format } from './format';
export class PopoverPosition {
    constructor(positionType, hostRect, popoverContent, resizeEntry, separatedPositionType, popoverSet) {
        this.positionType = positionType;
        this.hostRect = hostRect;
        this.popoverContent = popoverContent;
        this.resizeEntry = resizeEntry;
        this.separatedPositionType = separatedPositionType;
        this.popoverSet = popoverSet;
    }
    getPopoverRect() {
        return new Promise((resolve) => {
            requestAnimationFrame(() => {
                resolve(this.popoverContent.getBoundingClientRect());
            });
        });
    }
    async getResult() {
        await this.setPopoverRect();
        const [position, side] = this.separatedPositionType;
        // 📌positionResult = position that popover content will appear in screen (inject to translate)
        // 📌The Summary is the x / y position will place the popover on screen.
        const positionResult = this.getPosition();
        const positionChecked = this.checkPosition(position, positionResult);
        const sideChecked = this.checkSide(position, side, positionResult);
        // 📌true = position of popover is over the screen
        if (position !== positionChecked || side !== sideChecked) {
            this.setNewPositionType(positionChecked, sideChecked);
            return { translate: this.getPosition().translate, sideChecked, positionChecked };
        }
        return { translate: positionResult.translate, sideChecked, positionChecked };
    }
    setNewPositionType(positionChecked, sideChecked) {
        this.positionType = `${positionChecked}-${sideChecked}`;
    }
    checkSide(position, side, positionResult) {
        const { height, width } = this.resizeEntry.contentRect;
        const { x, y } = positionResult;
        const { height: popoverHeight, width: popoverWidth } = this.popoverRect;
        switch (side) {
            case 'center':
                return this.checkCenteredSide(position, x, y, width, popoverWidth, height, popoverHeight);
            case 'right':
                return this.checkRightSide(x);
            case 'left':
                return this.checkLeftSide(x, width, popoverWidth);
            case 'bottom':
                return this.checkBottomSide(y);
            case 'top':
                return this.checkTopSide(y, height, popoverHeight);
            default:
                return side;
        }
    }
    checkCenteredSide(position, x, y, width, popoverWidth, height, popoverHeight) {
        switch (position) {
            case 'left':
            case 'right':
                return y + popoverHeight > height ? 'bottom' : y < 0 ? 'top' : 'center';
            case 'top':
            case 'bottom':
                return x + popoverWidth > width ? 'right' : x < 0 ? 'left' : 'center';
            default:
                return 'center';
        }
    }
    checkRightSide(x) {
        return x < 0 ? 'left' : 'right';
    }
    checkLeftSide(x, width, popoverWidth) {
        return x + popoverWidth > width ? 'right' : 'left';
    }
    checkBottomSide(y) {
        return y < 0 ? 'top' : 'bottom';
    }
    checkTopSide(y, height, popoverHeight) {
        return y + popoverHeight > height ? 'bottom' : 'top';
    }
    checkPosition(position, positionResult) {
        const { height, width } = this.resizeEntry.contentRect;
        const { x, y } = positionResult;
        const { width: popoverWidth, height: popoverHeight } = this.popoverRect;
        switch (position) {
            case 'right':
                return popoverWidth + x > width ? 'left' : position;
            case 'left':
                return x < 0 ? 'right' : position;
            case 'bottom':
                return popoverHeight + y > height ? 'top' : position;
            case 'top':
                return y < 0 ? 'bottom' : position;
            default:
                return position;
        }
    }
    async setPopoverRect() {
        this.popoverRect = await this.getPopoverRect();
    }
    getPosition() {
        const hostRect = this.hostRect;
        const { width: hostWidth, height: hostHeight, left: hostLeft, right: hostRight, top: hostTop, bottom: hostBottom, } = hostRect;
        const popoverRect = this.popoverRect;
        const { width: popoverWidth, height: popoverHeight } = popoverRect;
        const positionType = this.positionType;
        let x, y;
        switch (positionType) {
            case 'bottom-left':
                x = hostLeft;
                y = hostBottom;
                break;
            case 'bottom-center':
                x = hostLeft + hostWidth / 2 - popoverWidth / 2;
                y = hostBottom;
                break;
            case 'bottom-right':
                x = hostRight - popoverWidth;
                y = hostBottom;
                break;
            case 'top-left':
                x = hostLeft;
                y = hostTop - popoverHeight;
                break;
            case 'top-center':
                x = hostLeft + hostWidth / 2 - popoverWidth / 2;
                y = hostTop - popoverHeight;
                break;
            case 'top-right':
                x = hostRight - popoverWidth;
                y = hostTop - popoverHeight;
                break;
            case 'left-top':
                x = hostLeft - popoverWidth;
                y = hostTop;
                break;
            case 'left-center':
                x = hostLeft - popoverWidth;
                y = hostTop + hostHeight / 2 - popoverHeight / 2;
                break;
            case 'left-bottom':
                x = hostLeft - popoverWidth;
                y = hostBottom - popoverHeight;
                break;
            case 'right-top':
                x = hostRight;
                y = hostTop;
                break;
            case 'right-center':
                x = hostRight;
                y = hostTop + hostHeight / 2 - popoverHeight / 2;
                break;
            case 'right-bottom':
                x = hostRight;
                y = hostBottom - popoverHeight;
                break;
            default:
                x = hostLeft;
                y = hostBottom;
                break;
        }
        const transformValue = this.checkTransform(positionType, { x, y }, { width: hostWidth, height: hostHeight }, this.popoverSet.transform);
        return format(transformValue);
    }
    checkTransform(position, value, host, transform) {
        switch (position) {
            case 'bottom-center':
            case 'bottom-left':
            case 'bottom-right':
                return {
                    x: value.x,
                    y: transform === 'center' ? value.y - host.height / 2 : value.y,
                };
            case 'top-center':
            case 'top-left':
            case 'top-right':
                return {
                    x: value.x,
                    y: transform === 'center' ? value.y + host.height / 2 : value.y,
                };
            case 'left-top':
            case 'left-center':
            case 'left-bottom':
                return {
                    y: value.y,
                    x: transform === 'center' ? value.x + host.width / 2 : value.x,
                };
            case 'right-top':
            case 'right-center':
            case 'right-bottom':
                return {
                    y: value.y,
                    x: transform === 'center' ? value.x - host.width / 2 : value.x,
                };
        }
    }
}
//# sourceMappingURL=PopoverPosition.js.map